@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
-from .utils import get_init_file, get_model_creator, dictify_params, parse_filenames, openers
+from .utils import get_init_file, dictify_params, parse_filenames, openers
 from betanegbinfit import distributions as dists
 from multiprocessing import Pool, cpu_count
 from statsmodels.stats import multitest
@@ -19,12 +19,13 @@ import os
 
 
 class Model():
-    def __init__(self, dist: str, left=4, mask_size=0, model_name='window'):
+    def __init__(self, dist: str, left=4, mask_size=0, model_name='window', param_mode='window'):
         self.dist = dist
         self.left = left
         self.allowed_const = left + 1
         self.mask = np.zeros(mask_size, dtype=bool)
         self.model_name = model_name
+        self.param_mode = param_mode
         self.grad = jax.grad(self.fun, argnums=0)
         self.fim = jax.jit(jax.grad(self.grad, argnums=0))
     
@@ -39,19 +40,26 @@ class Model():
         logl *= w
         return -jax.numpy.where(mask, 0.0, logl).sum()
     
-    def fit(self, data: np.ndarray, params: dict):
+    def fit(self, data: np.ndarray, params: dict, compute_var=True):
         name = self.model_name
         data, fixed, w = data.T
         n = len(data)
-        r = np.zeros(n, dtype=float)
-        if self.dist == 'BetaNB':
-            k = np.zeros(n, dtype=float)
-        for i in range(n):
-            slc = fixed[i]
-            ps = get_closest_param(params, slc, name, True)
-            r[i] = ps['r']
-            if 'k' in ps:
-                k[i] = ps['k']        
+        if self.param_mode == 'window' or self.model_name == 'slices':
+            r = np.zeros(n, dtype=float)
+            if self.dist == 'BetaNB':
+                k = np.zeros(n, dtype=float)
+            for i in range(n):
+                slc = fixed[i]
+                ps = get_closest_param(params, slc, name, True)
+                r[i] = ps['r']
+                if 'k' in ps:
+                    k[i] = ps['k']
+        else:
+            slc = fixed.mean()
+            ps = get_closest_param(params, slc, name, False)
+            r = ps['mu'] + ps['b'] * fixed ** ps.get('slice_power', 1.0)
+            if 'mu_k' in ps:
+                k = ps['mu_k'] + ps['b_k'] * np.log(fixed)
         mask = self.mask
         m = len(mask)
         if n > m:
@@ -71,7 +79,9 @@ class Model():
             fim = partial(self.fim, r=r, data=data, w=w, mask=mask)
         r = minimize_scalar(f, bounds=(0.0, 1.0), method='bounded')
         r.x = float(r.x)
-        return r, float(1 / fim(r.x))
+        if compute_var:
+            return r, float(1 / fim(r.x))
+        return r, -float(f(r.x))
         
     
 
@@ -144,47 +154,39 @@ def get_closest_param(params: dict, slc: float, model_name: str, compute_line=Fa
 
 
 def lrt_test(counts: Tuple[tuple, np.ndarray, np.ndarray, np.ndarray],
-        inst_params: dict, params: dict, skip_failures=False, max_sz=None, bad=1.0):
+        inst_params: dict, params: dict, skip_failures=False, max_sz=None, bad=1.0,
+        param_mode='window'):
     if not hasattr(lrt_test, '_cache'):
         lrt_test._cache = dict()
     snv, counts_a, counts_b, counts = counts
     cache = lrt_test._cache
-    key = tuple(inst_params.values())
-    name = inst_params['name']
+    key = (inst_params['dist'], inst_params['left'], max_sz if max_sz else 0, inst_params['name'], param_mode)
     if key not in cache:
-        inst_params = deepcopy(inst_params)
-        inst_params['name'] = 'line_diff'
-        inst_params['estimate_p'] = True
-        model = get_model_creator(**inst_params)()
-        x0 = np.zeros(model.num_params, dtype=float)
-        model.override_start([x0])
-        cache[key] = model, x0
+        model = Model(*key)
+        cache[key] = model
     else:
-        model, x0 = cache[key]
-    if max_sz is not None:
-        model.mask = np.zeros(max_sz, dtype=bool)
+        model = cache[key]
     res = list()
     for allele in ('ref', 'alt'):
         if allele == 'alt':
             counts_a = counts_a[:, (1, 0, 2)]; counts_b = counts_b[:, (1, 0, 2)]; counts = counts[:, (1, 0, 2)]
-        ps = get_closest_param(params[allele], counts[:, 1].mean(), name)
-        ps['p1'] = 1 / (bad + 1)
-        for p, v in ps.items():
-            model.set_param(p, x0, v)
         try:
-            unified = model.fit(counts, use_prev=False, )
-            a = model.fit(counts_a, use_prev=False, )
-            b = model.fit(counts_b, use_prev=False, )
-            success = a['success'] & b['success'] & unified['success']
+            ab_r, ab_logl = model.fit(counts, params[allele], False)
+            ab_p = ab_r.x
+            a_r, a_logl = model.fit(counts_a, params[allele], False)
+            a_p = a_r.x
+            b_r, b_logl = model.fit(counts_b, params[allele], False)
+            b_p = b_r.x
+            success = a_r.success & b_r.success & ab_r.success
             if not success and skip_failures:
                 return None, snv
         except (KeyError, IndexError):
             if skip_failures:
                 return None, snv
             res.append((float('nan'), float('nan'), float('nan'), float('nan')))
-        lrt = -2 * (unified['loglik'] - (a['loglik'] + b['loglik']))
+        lrt = -2 * (ab_logl - (a_logl + b_logl))
         pval = chi2.sf(lrt, 1)
-        res.append((pval, unified['p1'], a['p1'], b['p1']))
+        res.append((pval, ab_p, a_p, b_p))
     n_a, n_b = counts_a[:, -1].sum(), counts_b[:, -1].sum()
     return res, (snv, n_a, n_b)
 
@@ -199,19 +201,21 @@ def calc_var(m, data: np.ndarray, renormalize=None):
         var *= n / renormalize
     return float(var)
 
+
 def transform_p(p, var):
     pt = np.log(p) - np.log1p(-p)
-    var = var * (1 / p - 1 / (1 - p)) ** 2
+    var = var * (1 / p + 1 / (1 - p)) ** 2
     return pt, var
 
 def wald_test(counts: Tuple[tuple, np.ndarray, np.ndarray, np.ndarray],
               inst_params: dict, params: dict, skip_failures=False, max_sz=None, bad=1.0,
-              contrast: Tuple[float, float, float] = (1, -1, 0)):
+              contrast: Tuple[float, float, float] = (1, -1, 0), logit_transform=False,
+              param_mode='window'):
     if not hasattr(wald_test, '_cache'):
         wald_test._cache = dict()
     snv, counts_a, counts_b, counts = counts
     cache = wald_test._cache
-    key = (inst_params['dist'], inst_params['left'], max_sz if max_sz else 0, inst_params['name'])
+    key = (inst_params['dist'], inst_params['left'], max_sz if max_sz else 0, inst_params['name'], param_mode)
     if key not in cache:
         model = Model(*key)
         cache[key] = model
@@ -227,6 +231,9 @@ def wald_test(counts: Tuple[tuple, np.ndarray, np.ndarray, np.ndarray],
             b_r, b_var = model.fit(counts_b, params[allele])
             b_p = b_r.x
             correct = (a_var >= 0) & (b_var >= 0) & np.isfinite(a_var) & np.isfinite(b_var)
+            if logit_transform:
+                a_p, a_var = transform_p(a_p, a_var)
+                b_p, b_var = transform_p(b_p, b_var)
             success = a_r.success & b_r.success & correct
             if not success and skip_failures:
                 return None, snv
@@ -238,14 +245,18 @@ def wald_test(counts: Tuple[tuple, np.ndarray, np.ndarray, np.ndarray],
         if not correct:
             pval = 1.0
         else:
-            pval = 2 * norm.sf(abs(a_p - b_p), scale=(a_var + b_var) ** 0.5, loc=0.0)
+            a, b, eq = contrast
+            stat = abs(a * a_p + b * b_p + eq)
+            var = a ** 2 * a_var + b ** 2 * b_var
+            pval = 2 * norm.sf(stat, scale=(var) ** 0.5, loc=0.0)
         res.append((pval, a_p, b_p, a_var ** 0.5, b_var ** 0.5))
     n_a, n_b = counts_a[:, -1].sum(), counts_b[:, -1].sum()
     return res, (snv, n_a, n_b)
         
 def differential_test(name: str, group_a: List[str], group_b: List[str], mode='wald', min_samples=2, min_cover=0,
                       max_cover=np.inf, skip_failures=True, group_test=True, alpha=0.05, max_cover_group_test=None,
-                      filter_chr=None, filter_id=None, contrast=(1, -1, 0), subname=None, n_jobs=-1):  
+                      filter_chr=None, filter_id=None, contrast=(1, -1, 0), subname=None, param_mode='window',
+                      logit_transform=False, n_jobs=-1):  
     if max_cover is None:
         max_cover = np.inf
     if min_cover is None:
@@ -290,12 +301,13 @@ def differential_test(name: str, group_a: List[str], group_b: List[str], mode='w
     if mode == 'lrt':
         cols = ['ref_pval', 'ref_p_ab', 'ref_p_a', 'ref_p_b', 
                 'alt_pval', 'alt_p_ab', 'alt_p_a', 'alt_p_b']
-        test_fun = partial(lrt_test, inst_params=inst_params, params=params, skip_failures=False, bad=bad)
+        test_fun = partial(lrt_test, inst_params=inst_params, params=params, skip_failures=False, bad=bad,
+                           param_mode=param_mode)
     else:
         cols = ['ref_pval', 'ref_p_a', 'ref_p_b', 'ref_std_a', 'ref_std_b',
                 'alt_pval', 'alt_p_a', 'alt_p_b', 'alt_std_a', 'alt_std_b']
         test_fun = partial(wald_test, inst_params=inst_params, params=params, skip_failures=False, bad=bad,
-                           contrast=contrast)
+                           contrast=contrast, param_mode=param_mode, logit_transform=logit_transform)
     if group_test:
         counts_a = _counts_a[bad]; counts_b = _counts_b[bad]; counts = _counts[bad]
         counts_a = counts_a[counts_a[:, 0] + counts_a[:, 1] < max_cover_group_test]
